@@ -45,8 +45,8 @@
 
 #define SAY(format,...) do {\
 	if (strcmp("Debug", BUILDTYPE)==0) {\
-	    fwide(stderr, -1);\
 	    fprintf(stderr, "[%d] ", getpid());\
+	    fprintf(stderr, "[%s] ", __FUNCTION__);\
 	    if (format) fprintf(stderr, format, ##__VA_ARGS__);\
 	    fputc('\n', stderr);\
 		fflush(stderr);\
@@ -136,7 +136,6 @@ const GdkColor rxvt_palette[PALETTE_SIZE] =
 	{ 0, 0xffff, 0xffff, 0xffff }
 };
 
-
 static struct {
 	GtkWidget *main_window;
 	GtkWidget *notebook;
@@ -148,12 +147,10 @@ static struct {
 	char *current_match;
 	guint width;
 	guint height;
-	struct {
-		glong columns;
-		glong rows;
-		gint char_width;
-		gint char_height;
-	} term_info;
+	glong columns;
+	glong rows;
+	gint char_width;
+	gint char_height;
 	gint label_count;
 	bool fake_transparency;
 	float opacity_level;
@@ -164,7 +161,9 @@ static struct {
 	bool show_closebutton;
 	bool audible_bell;
 	bool visible_bell;
+	bool blinking_cursor;
 	bool full_screen;
+	bool keep_fc; /* Global flag to indicate that we don't want changes in the files and columns values */
 	GtkWidget *item_clear_background; /* We include here only the items which need to be hided */
 	GtkWidget *item_copy_link;
 	GtkWidget *item_open_link;
@@ -188,6 +187,7 @@ static struct {
 	gint paste_key;
 	gint scrollbar_key;
 	gint fullscreen_key;
+	GRegex *http_regexp;
 	char *argv[2];
 } sakura;
 
@@ -200,9 +200,9 @@ struct terminal {
 };
 
 
-#define ICON_DIR "/usr/share/pixmaps"
+#define ICON_FILE "terminal-tango.svg"
 #define SCROLL_LINES 4096
-#define HTTP_REGEXP "((f|F)(t|T)(p|P)|((h|H)(t|T)(t|T)(p|P)(s|S)*))://[-a-zA-Z0-9.?$%&/=_~#.,:;+]*"
+#define HTTP_REGEXP "(ftp|http)s?://[-a-zA-Z0-9.?$%&/=_~#.,:;+]*"
 #define CONFIGFILE "sakura.conf"
 #define DEFAULT_COLUMNS 80
 #define DEFAULT_ROWS 24
@@ -274,11 +274,11 @@ static void     sakura_destroy();
 static void     sakura_add_tab();
 static void     sakura_del_tab();
 static void     sakura_set_font();
-static void     sakura_set_size();
+static void     sakura_set_size(gint, gint);
 static void     sakura_kill_child();
 static void     sakura_set_bgimage();
-static void     sakura_key_file_set_key(GKeyFile *cfg,const gchar *cfg_group,const gchar *keyname,guint value);
-static guint    sakura_key_file_get_key(GKeyFile *cfg,const gchar *cfg_group,const gchar *keyname);
+static void     sakura_key_file_set_key(GKeyFile *, const gchar *, const gchar *, guint);
+static guint    sakura_key_file_get_key(GKeyFile *, const gchar *, const gchar *);
 
 static const char *option_font;
 static const char *option_execute;
@@ -376,6 +376,7 @@ gboolean sakura_key_press (GtkWidget *widget, GdkEventKey *event, gpointer user_
 	if ( (event->state & sakura.scrollbar_accelerator)==sakura.scrollbar_accelerator ) {
 		if (event->keyval==sakura.scrollbar_key) {
 			sakura_show_scrollbar(NULL, NULL);
+			sakura_set_size(sakura.columns, sakura.rows);
 			return TRUE;
 		}
 	}
@@ -401,7 +402,6 @@ sakura_button_press(GtkWidget *widget, GdkEventButton *button_event, gpointer us
 		return FALSE;
 
 	page = gtk_notebook_get_current_page(GTK_NOTEBOOK(sakura.notebook));
-	SAY("Current page is %d", page);
 	term = sakura_get_page_term(sakura, page);
 
 	/* Find out if cursor it's over a matched expression...*/
@@ -455,7 +455,7 @@ sakura_page_removed (GtkWidget *widget, void *data)
 	if (gtk_notebook_get_n_pages(GTK_NOTEBOOK(sakura.notebook))==1) {
 		/* If the first tab is disabled, window size changes and we need
 		 * to recalculate its size */
-		sakura_set_size();
+		sakura_set_size(sakura.columns, sakura.rows);
 	}
 }
 
@@ -469,6 +469,7 @@ sakura_increase_font (GtkWidget *widget, void *data)
 	pango_font_description_set_size(sakura.font, ((size/PANGO_SCALE)+1) * PANGO_SCALE);
 
 	sakura_set_font();
+	sakura_set_size(sakura.columns, sakura.rows);
 }
 
 
@@ -481,6 +482,7 @@ sakura_decrease_font (GtkWidget *widget, void *data)
 	pango_font_description_set_size(sakura.font, ((size/PANGO_SCALE)-1) * PANGO_SCALE);
 
 	sakura_set_font();
+	sakura_set_size(sakura.columns, sakura.rows);
 }
 
 
@@ -565,11 +567,12 @@ sakura_delete_window (GtkWidget *widget, void *data)
 {
 	GtkWidget *dialog;
 	guint response;
+	gint npages=gtk_notebook_get_n_pages(GTK_NOTEBOOK(sakura.notebook));
 
-	if (gtk_notebook_get_n_pages(GTK_NOTEBOOK(sakura.notebook))>1) {
+	if (npages > 1) {
 		dialog=gtk_message_dialog_new(GTK_WINDOW(sakura.main_window), GTK_DIALOG_MODAL,
 									  GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO,
-									  _("There are several tabs opened. Do you really want to close Sakura?"));
+									  _("There are %d tabs opened.\n\nDo you really want to close Sakura?"), npages);
 
 		response=gtk_dialog_run(GTK_DIALOG(dialog));
 		gtk_widget_destroy(dialog);
@@ -608,6 +611,7 @@ sakura_font_dialog (GtkWidget *widget, void *data)
 		pango_font_description_free(sakura.font);
 		sakura.font=pango_font_description_from_string(gtk_font_selection_dialog_get_font_name(GTK_FONT_SELECTION_DIALOG(font_dialog)));
 		sakura_set_font();
+		sakura_set_size(sakura.columns, sakura.rows);
 		g_key_file_set_value(sakura.cfg, cfg_group, "font", pango_font_description_to_string(sakura.font));
 	}
 
@@ -1017,6 +1021,8 @@ sakura_show_scrollbar (GtkWidget *widget, void *data)
 	int n_pages;
 	int i;
 
+	sakura.keep_fc=1;
+
 	n_pages=gtk_notebook_get_n_pages(GTK_NOTEBOOK(sakura.notebook));
 	page = gtk_notebook_get_current_page(GTK_NOTEBOOK(sakura.notebook));
 	term = sakura_get_page_term(sakura, page);
@@ -1078,6 +1084,25 @@ sakura_visible_bell (GtkWidget *widget, void *data)
 }
 
 
+static void
+sakura_blinking_cursor (GtkWidget *widget, void *data)
+{
+	int page;
+	struct terminal *term;
+
+	page = gtk_notebook_get_current_page(GTK_NOTEBOOK(sakura.notebook));
+	term = sakura_get_page_term(sakura, page);
+
+	if (gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(widget))) {
+		vte_terminal_set_cursor_blink_mode (VTE_TERMINAL(term->vte), VTE_CURSOR_BLINK_ON);
+		g_key_file_set_value(sakura.cfg, cfg_group, "blinking_cursor", "Yes");
+	} else {
+		vte_terminal_set_cursor_blink_mode (VTE_TERMINAL(term->vte), VTE_CURSOR_BLINK_OFF);
+		g_key_file_set_value(sakura.cfg, cfg_group, "blinking_cursor", "No");
+	}
+}
+
+
 static void
 sakura_set_palette(GtkWidget *widget, void *data)
 {
@@ -1110,30 +1135,38 @@ sakura_set_palette(GtkWidget *widget, void *data)
 }
 
 
-/* Every the window change its size by an user action (resize, fullscreen), calculate
+/* Every the window changes its size by an user action (resize, fullscreen), calculate
  * the new values for the number of columns and rows */
 static void
 sakura_calculate_row_col (gint width, gint height)
 {
 	struct terminal *term;
-	gint x_adjust, y_adjust;
+	gint x_padding, y_padding;
 	gint n_pages=gtk_notebook_get_n_pages(GTK_NOTEBOOK(sakura.notebook));
 
 	if (n_pages==-1) return;
 
+	SAY("Calculating row_col");
 	term = sakura_get_page_term(sakura, 0);
 
 	/* This is to prevent a race with ConfigureEvents when the window is being destroyed */
 	if (!VTE_IS_TERMINAL(term->vte)) return;
 
-	vte_terminal_get_padding( VTE_TERMINAL(term->vte), &x_adjust, &y_adjust );
-	sakura.term_info.char_width = vte_terminal_get_char_width(VTE_TERMINAL(term->vte));
-	sakura.term_info.char_height = vte_terminal_get_char_height(VTE_TERMINAL(term->vte));
-	x_adjust += sakura.main_window->allocation.width - term->vte->allocation.width;
-	sakura.term_info.columns = (width - x_adjust) / sakura.term_info.char_width;
-	y_adjust += sakura.main_window->allocation.height - term->vte->allocation.height;
-	sakura.term_info.rows = (height - y_adjust) / sakura.term_info.char_height;
-	SAY("columns %ld rows %ld", sakura.term_info.columns, sakura.term_info.rows);
+	vte_terminal_get_padding( VTE_TERMINAL(term->vte), &x_padding, &y_padding );
+	sakura.char_width = vte_terminal_get_char_width(VTE_TERMINAL(term->vte));
+	sakura.char_height = vte_terminal_get_char_height(VTE_TERMINAL(term->vte));
+	/* Ignore resize events in sakura window is in fullscreen */
+	if (!sakura.keep_fc) {	
+		/* We cannot trust in vte allocation values, they're unreliable */
+		/* FIXME: Round values properly */
+		sakura.columns = (width/sakura.char_width);
+		sakura.rows = (height/sakura.char_height);
+		sakura.keep_fc=false;
+		SAY("new columns %ld and rows %ld", sakura.columns, sakura.rows);
+	}
+	sakura.width = sakura.main_window->allocation.width + x_padding;
+	sakura.height = sakura.main_window->allocation.height + y_padding;
+	//}
 }
 
 
@@ -1168,14 +1201,11 @@ sakura_get_term_cwd(struct terminal* term)
 static gboolean
 sakura_resized_window (GtkWidget *widget, GdkEventConfigure *event, void *data)
 {
-	SAY("resized event received");
 	if (event->width!=sakura.width || event->height!=sakura.height) {
 		SAY("sakura w & h %d %d event w & h %d %d",
 		sakura.width, sakura.height, event->width, event->height);
 		/* Window has been resized by the user. Recalculate sizes */
 		sakura_calculate_row_col (event->width, event->height);
-		/* FIXME: Should it be inside row_col */
-		sakura.width=event->width; sakura.height=event->height;
 	} 
 
 	return FALSE;
@@ -1246,8 +1276,8 @@ static void
 sakura_full_screen (GtkWidget *widget, void *data)
 {
 	if (sakura.full_screen!=TRUE) {
-		gtk_window_fullscreen(GTK_WINDOW(sakura.main_window));
 		sakura.full_screen=TRUE;
+		gtk_window_fullscreen(GTK_WINDOW(sakura.main_window));
 	} else {
 		gtk_window_unfullscreen(GTK_WINDOW(sakura.main_window));
 		sakura.full_screen=FALSE;
@@ -1401,6 +1431,13 @@ sakura_init()
 	sakura.visible_bell= (strcmp(cfgtmp, "Yes")==0) ? 1 : 0;
 	g_free(cfgtmp);
 
+	if (!g_key_file_has_key(sakura.cfg, cfg_group, "blinking_cursor", NULL)) {
+		g_key_file_set_value(sakura.cfg, cfg_group, "blinking_cursor", "No");
+	}
+	cfgtmp = g_key_file_get_value(sakura.cfg, cfg_group, "blinking_cursor", NULL);
+	sakura.blinking_cursor= (strcmp(cfgtmp, "Yes")==0) ? 1 : 0;
+	g_free(cfgtmp);
+
 	if (!g_key_file_has_key(sakura.cfg, cfg_group, "word_chars", NULL)) {
 		g_key_file_set_value(sakura.cfg, cfg_group, "word_chars", DEFAULT_WORD_CHARS);
 	}
@@ -1499,10 +1536,10 @@ sakura_init()
 
 	sakura.main_window=gtk_window_new(GTK_WINDOW_TOPLEVEL);
 	gtk_window_set_title(GTK_WINDOW(sakura.main_window), "sakura");
-	gtk_window_set_icon_from_file(GTK_WINDOW(sakura.main_window), ICON_DIR "/terminal-tango.png", &gerror);
-	/* Minimum size*/
-	sakura.term_info.columns = DEFAULT_COLUMNS;
-	sakura.term_info.rows = DEFAULT_ROWS;
+	gtk_window_set_icon_from_file(GTK_WINDOW(sakura.main_window), DATADIR "/pixmaps/" ICON_FILE, &gerror);
+	/* Default terminal size*/
+	sakura.columns = DEFAULT_COLUMNS;
+	sakura.rows = DEFAULT_ROWS;
 
 	sakura.notebook=gtk_notebook_new();
 
@@ -1519,11 +1556,11 @@ sakura_init()
 	}
 
 	if (option_columns) {
-		sakura.term_info.columns = option_columns;
+		sakura.columns = option_columns;
 	}
 
 	if (option_rows) {
-		sakura.term_info.rows = option_rows;
+		sakura.rows = option_rows;
 	}
 
 	if (option_font) {
@@ -1537,6 +1574,10 @@ sakura_init()
 	sakura.menu=gtk_menu_new();
 	sakura.label_count=1;
 	sakura.full_screen=FALSE;
+	sakura.keep_fc=false;
+
+	gerror=NULL;
+	sakura.http_regexp=g_regex_new(HTTP_REGEXP, G_REGEX_CASELESS, G_REGEX_MATCH_NOTEMPTY, &gerror);
 
 	gtk_container_add(GTK_CONTAINER(sakura.main_window), sakura.notebook);
 
@@ -1560,6 +1601,7 @@ sakura_init_popup()
 	          *item_select_background, *item_set_title, *item_full_screen,
 	          *item_toggle_scrollbar, *item_options, *item_input_methods,
 	          *item_opacity_menu, *item_show_first_tab, *item_audible_bell, *item_visible_bell,
+                  *item_blinking_cursor,
 	          *item_palette, *item_palette_tango, *item_palette_linux, *item_palette_xterm, *item_palette_rxvt,
 	          *item_show_close_button;
 	GtkAction *action_open_link, *action_copy_link, *action_new_tab, *action_set_name, *action_close_tab,
@@ -1605,6 +1647,7 @@ sakura_init_popup()
 	item_toggle_scrollbar=gtk_check_menu_item_new_with_label(_("Toggle scrollbar"));
 	item_audible_bell=gtk_check_menu_item_new_with_label(_("Set audible bell"));
 	item_visible_bell=gtk_check_menu_item_new_with_label(_("Set visible bell"));
+	item_blinking_cursor=gtk_check_menu_item_new_with_label(_("Set blinking cursor"));
 	item_input_methods=gtk_menu_item_new_with_label(_("Input methods"));
 	item_palette_tango=gtk_radio_menu_item_new_with_label(NULL, "Tango");
 	item_palette_linux=gtk_radio_menu_item_new_with_label_from_widget(GTK_RADIO_MENU_ITEM(item_palette_tango), "Linux");
@@ -1644,6 +1687,10 @@ sakura_init_popup()
 		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item_visible_bell), TRUE);
 	}
 
+	if (sakura.blinking_cursor) {
+		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item_blinking_cursor), TRUE);
+	}
+
 	cfgtmp = g_key_file_get_string(sakura.cfg, cfg_group, "palette", NULL);
 	if (strcmp(cfgtmp, "linux")==0) {
 		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item_palette_linux), TRUE);
@@ -1689,6 +1736,7 @@ sakura_init_popup()
 	gtk_menu_shell_append(GTK_MENU_SHELL(options_menu), item_toggle_scrollbar);
 	gtk_menu_shell_append(GTK_MENU_SHELL(options_menu), item_audible_bell);
 	gtk_menu_shell_append(GTK_MENU_SHELL(options_menu), item_visible_bell);
+	gtk_menu_shell_append(GTK_MENU_SHELL(options_menu), item_blinking_cursor);
 	gtk_menu_shell_append(GTK_MENU_SHELL(options_menu), gtk_separator_menu_item_new());
 	gtk_menu_shell_append(GTK_MENU_SHELL(options_menu), item_opacity_menu);
 	gtk_menu_shell_append(GTK_MENU_SHELL(options_menu), item_set_title);
@@ -1718,6 +1766,7 @@ sakura_init_popup()
 	g_signal_connect(G_OBJECT(item_toggle_scrollbar), "activate", G_CALLBACK(sakura_show_scrollbar), NULL);
 	g_signal_connect(G_OBJECT(item_audible_bell), "activate", G_CALLBACK(sakura_audible_bell), NULL);
 	g_signal_connect(G_OBJECT(item_visible_bell), "activate", G_CALLBACK(sakura_visible_bell), NULL);
+	g_signal_connect(G_OBJECT(item_blinking_cursor), "activate", G_CALLBACK(sakura_blinking_cursor), NULL);
 	g_signal_connect(G_OBJECT(action_opacity), "activate", G_CALLBACK(sakura_opacity_dialog), NULL);
 	g_signal_connect(G_OBJECT(action_set_title), "activate", G_CALLBACK(sakura_set_title_dialog), NULL);
 	g_signal_connect(G_OBJECT(item_palette_tango), "activate", G_CALLBACK(sakura_set_palette), "tango");
@@ -1791,7 +1840,7 @@ sakura_destroy()
 
 
 static void
-sakura_set_size()
+sakura_set_size(gint columns, gint rows)
 {
 	struct terminal *term;
 	GtkRequisition main_request;
@@ -1801,6 +1850,11 @@ sakura_set_size()
 	gint char_width, char_height;
 
 	term = sakura_get_page_term(sakura, 0);
+
+	/* New values used to resize the window */
+	//sakura.columns = columns;
+	//sakura.rows = rows;
+
 	vte_terminal_get_padding(VTE_TERMINAL(term->vte), (int *)&pad_x, (int *)&pad_y);
 	char_width = vte_terminal_get_char_width(VTE_TERMINAL(term->vte));
 	char_height = vte_terminal_get_char_height(VTE_TERMINAL(term->vte));
@@ -1820,10 +1874,11 @@ sakura_set_size()
 	gtk_widget_size_request (term->vte, &term_request);
 	sakura.width = main_request.width - term_request.width;
 	sakura.height = main_request.height - term_request.height;
-	sakura.width += pad_x + char_width * sakura.term_info.columns;
-	sakura.height += pad_y + char_height * sakura.term_info.rows;
+	sakura.width += pad_x + char_width * sakura.columns;
+	sakura.height += pad_y + char_height * sakura.rows;
 	if (GTK_WIDGET_MAPPED (sakura.main_window)) {
 		gtk_window_resize (GTK_WINDOW (sakura.main_window), sakura.width, sakura.height);
+		SAY("Resizing to %ld columns %ld rows", sakura.columns, sakura.rows);
 	} else {
 		gtk_window_set_default_size (GTK_WINDOW (sakura.main_window), sakura.width, sakura.height);
 	}
@@ -1863,8 +1918,6 @@ sakura_add_tab()
 	term->hbox=gtk_hbox_new(FALSE, 0);
 	term->vte=vte_terminal_new();
 
-	vte_terminal_set_size(VTE_TERMINAL(term->vte), DEFAULT_COLUMNS, DEFAULT_ROWS);
-
 	/* Create label (and optional close button) for tabs */
 	label_text=g_strdup_printf(_("Terminal %d"), sakura.label_count++);
 	term->label=gtk_label_new(label_text);
@@ -1885,7 +1938,7 @@ sakura_add_tab()
 
 	/* Init vte */
 	vte_terminal_set_scrollback_lines(VTE_TERMINAL(term->vte), SCROLL_LINES);
-	vte_terminal_match_add(VTE_TERMINAL(term->vte), HTTP_REGEXP);
+	vte_terminal_match_add_gregex(VTE_TERMINAL(term->vte), sakura.http_regexp, 0);
 	vte_terminal_set_mouse_autohide(VTE_TERMINAL(term->vte), TRUE);
 
 	term->scrollbar=gtk_vscrollbar_new(vte_terminal_get_adjustment(VTE_TERMINAL(term->vte)));
@@ -1903,6 +1956,9 @@ sakura_add_tab()
 	if (!cwd)
 		cwd = g_get_current_dir();
 
+	/* Keep values when adding tabs */
+	sakura.keep_fc=true;
+
 	if ((index=gtk_notebook_append_page(GTK_NOTEBOOK(sakura.notebook), term->hbox, tab_hbox))==-1) {
 		sakura_error("Cannot create a new tab");
 		return;
@@ -1945,7 +2001,8 @@ sakura_add_tab()
 		gtk_notebook_set_show_border(GTK_NOTEBOOK(sakura.notebook), FALSE);
 		sakura_set_font();
 		gtk_widget_show_all(sakura.main_window);
-		sakura_set_size();
+		/* We need only to call set_size for the first tab */
+		sakura_set_size(sakura.columns, sakura.rows);
 
 		if (option_execute) {
 			int command_argc; char **command_argv;
@@ -1982,7 +2039,7 @@ sakura_add_tab()
 		gtk_notebook_set_show_tabs(GTK_NOTEBOOK(sakura.notebook), TRUE);
 		sakura_set_font();
 		gtk_widget_show_all(term->hbox);
-		sakura_set_size();
+		sakura_set_size(sakura.columns, sakura.rows);
 		/* Call set_current page after showing the widget: gtk ignores this
 		 * function in the window is not visible *sigh*. Gtk documentation
 		 * says this is for "historical" reasons. Me arse */
@@ -1993,7 +2050,11 @@ sakura_add_tab()
 
 	free(cwd);
 
-	if (!sakura.show_scrollbar) gtk_widget_hide(term->scrollbar);
+	/* Recalculate sizes if the scrollbar is disabled */
+	if (!sakura.show_scrollbar) {
+		gtk_widget_hide(term->scrollbar);
+		sakura_set_size(sakura.columns, sakura.rows);
+	}
 
 	/* Configuration per-terminal */
 	vte_terminal_set_backspace_binding(VTE_TERMINAL(term->vte), VTE_ERASE_ASCII_DELETE);
@@ -2017,8 +2078,16 @@ sakura_add_tab()
 	vte_terminal_set_audible_bell (VTE_TERMINAL(term->vte), sakura.audible_bell ? TRUE : FALSE);
 	vte_terminal_set_visible_bell (VTE_TERMINAL(term->vte), sakura.visible_bell ? TRUE : FALSE);
 
+	/* Disable stupid blinking cursor */
+	vte_terminal_set_cursor_blink_mode (VTE_TERMINAL(term->vte), sakura.blinking_cursor ? VTE_CURSOR_BLINK_ON : VTE_CURSOR_BLINK_OFF);
+
+
 	/* Grrrr. Why the fucking label widget in the notebook STEAL the fucking focus? */
 	gtk_widget_grab_focus(term->vte);
+
+	/* FIXME: Possible race here. Find some way to force to process all configure
+	 * events before setting keep_fc again to false */
+	sakura.keep_fc=false;
 }
 
 
@@ -2039,6 +2108,7 @@ sakura_del_tab(gint page)
 			gtk_notebook_set_show_tabs(GTK_NOTEBOOK(sakura.notebook), FALSE);
 		}
 		g_free(cfgtmp);
+		sakura.keep_fc=true;
 	}
 
 	gtk_widget_hide(term->hbox);
